// Hyperbolic Rogue for Android
// Copyright (C) 2012 Zeno Rogue

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#define VER "3.7"
#define ISANDROID 1

#ifdef FAKE
#define ANDROID
#endif

#include <math.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>

#include "stl.cpp"

using namespace std;

typedef double ld;

template<class T> int size(T x) {return x.size(); }
string its(int i) { char buf[64]; sprintf(buf, "%d", i); return buf; }
string fts(float x) { char buf[64]; sprintf(buf, "%4.2f", x); return buf; }

string s0;
void addMessage(string s);

#include "src/hyperpoint.cpp"
#include "src/heptagon.cpp"
#include "src/classes.cpp"
#include "src/cell.cpp"
#include "src/game.cpp"
#include "src/graph.cpp"

void panning(hyperpoint hf, hyperpoint ht) {

  View = 
    rgpushxto0(hf) * rgpushxto0(gpushxto0(hf) * ht) * gpushxto0(hf) * View;
  playermoved = false;
  }

#ifdef FAKE

typedef int JNIEnv;
typedef int jobject;
typedef int jint;
typedef int jboolean;
typedef char jchar;
#define delref ;

#else

#define delref env->DeleteLocalRef(_thiz)

#include <jni.h>
extern "C" jintArray
Java_com_roguetemple_hyperroid_HyperRogue_loadMap
  ( JNIEnv* env, jobject _thiz)
{
    jintArray result;
    result = env->NewIntArray(size(graphdata));
    if(result == NULL) return NULL;
    
    env->SetIntArrayRegion(result, 0, size(graphdata), graphdata.begin());
    env->DeleteLocalRef(_thiz);
    env->DeleteLocalRef(result);
    return result;
}

#endif

extern "C" 
jint
Java_com_roguetemple_hyperroid_HyperRogue_initGame( JNIEnv* env,
                                                  jobject _thiz ) {
  srand(time(NULL));
  delref;
  
  initcells();
  initgame();
  initgraph();
  sightrange = 5;
  
  return 0;
  }
    
void showHelp(JNIEnv* env, jobject thiz, string& help) {
#ifdef FAKE
  printf("%s\n", help.c_str());
#else
  jclass cls = env->GetObjectClass(thiz);
  jmethodID mid = env->GetMethodID(cls, "showHelp", "(Ljava/lang/String;)V");
  jobject str = env->NewStringUTF(help.c_str());
  env->CallVoidMethod(thiz, mid, str);
  env->DeleteLocalRef(str);
  env->DeleteLocalRef(cls);
#endif
  }
  
void shareScore(JNIEnv* env, jobject thiz) {
  string s;
  time_t timer;
  timer = time(NULL);
  char buf[128]; strftime(buf, 128, "%c", localtime(&timer));
  char buf2[128];
  
  s += "HyperRogue III for Android ("VER"), http://www.roguetemple.com/hyper.php\n";
  sprintf(buf2, "Date: %s distance: %d\n", buf, celldist(cwt.c));
  s += buf2;
  s += "Score: " + its(gold());

  for(int i=0; i<firstorb; i++) if(items[i]) {
    sprintf(buf2, " %s (%d)", iinf[i].name, items[i]);
    s += buf2;
    }
  s += "\n";
  s += "Kills: " + its(tkills());
  for(int i=1; i<motypes; i++) if(kills[i]) {
    sprintf(buf2, " %s (%d)", minf[i].name, kills[i]);
    s += buf2;
    }
  s += "\n";

#ifdef FAKE
  printf("%s\n", s.c_str());
#else
  jclass cls = env->GetObjectClass(thiz);
  jmethodID mid = env->GetMethodID(cls, "shareScore", "(Ljava/lang/String;)V");
  jobject str = env->NewStringUTF(s.c_str());
  env->CallVoidMethod(thiz, mid, str);
  env->DeleteLocalRef(str);
  env->DeleteLocalRef(cls);
#endif
  }

bool lclicked = false;
string lmouseovers;

extern "C" void Java_com_roguetemple_hyperroid_HyperRogue_update
  ( JNIEnv* env, jobject _thiz, 
    jint xres, jint yres, jint _ticks,
    jint _mousex, jint _mousey, jboolean clicked,
    jchar action) {
  
  if(action == 'a') vid.wallmode = 0, vid.monmode = 0;
  if(action == 'b') vid.wallmode = 1, vid.monmode = 2;
  if(action == 'p') vid.wallmode = 2, vid.monmode = 2;
  if(action == 'e') vid.wallmode = 3, vid.monmode = 2;
  if(action == 'K') vid.alpha = 0;
  if(action == 'P') vid.alpha = 1;
  if(action == 'X') vid.alpha = 2;
  if(action == 'f') sightrange ^= 2;
  if(action == 'w') revcontrol = !revcontrol;
  if(action == 's') shareScore(env, _thiz);
  if(action == 'h') {
    help = helptext; showHelp(env,_thiz,help);
    }
  if(action == 'q') {
    if(cmode == emQuit) cmode = emNormal;
    else cmode = emQuit;
    }
  if(action == 'r') {
    cmode = emNormal; andmode = 0;
    restartGame();
    }
  
  vid.xres = xres;
  vid.yres = yres;
  vid.fsize = vid.xres / 32;
  
  mousex = _mousex;
  mousey = _mousey;
  
  hyperpoint mouseoh = mouseh;
  gtouched = clicked;
  lmouseover = (gtouched && lclicked) ? mouseover : NULL;
  if(andmode == 0 && !items[itOrbTeleport] && gtouched && lclicked) {
    lmouseover = mousedest >= 0 ? cwt.c->mov[mousedest] : cwt.c;
    }
  mouseh = gethyper(mousex, mousey);
  
  frames++;
  optimizeview();

  int lastt = ticks; ticks = _ticks;
  if(lastt > ticks) lastt = ticks;
  if(playermoved && vid.aspeed > -4.99)
    centerpc((ticks - lastt) / 1000.0 * exp(vid.aspeed));

  if(lclicked && !clicked) {

    if(mouseh[2] < .5) {
      
      if(mousey > vid.ycenter - vid.radius &&
         mousey < vid.ycenter + vid.radius &&
         mousex > vid.xcenter - vid.radius &&
         mousex < vid.xcenter + vid.radius) {
        int px = mousex < vid.xcenter ? 0 : 1;
        int py = mousey < vid.ycenter ? 0 : 1;
        
        if(cmode == emNormal) {
          if(px == 0 && py == 0) andmode = 10;
          if(px == 1 && py == 0) {
            if(andmode == 1) {
              centerpc(INF);
              View = Id;
              viewctr.h = cwt.c->master;
              }
            andmode = 11;
            }
          if(px == 0 && py == 1) andmode = 12;
          if(px == 1 && py == 1) andmode = andmode == 3 ? 4 : 13;
          }
        
        else if(cmode == emQuit) {
          if(px == 1 && py == 1) {
            if(canmove)
              cmode = emNormal, andmode = 0;
            else shareScore(env, _thiz);
            }
          if(px == 0 && py == 1) restartGame(), cmode = emNormal, andmode = 0;
          }
        }
      else if(andmode == 0 && getcstat == 'g') {
        movepcto(-2);
        }
      
      else {
        if(andmode == 0 && help != helptext) {
          addMessage(mouseovers);
          showHelp(env, _thiz, help);
          }
        }
      }
    
    if(andmode == 0 && cmode == emNormal && mouseh[2] > .5) {
      if(items[itOrbTeleport] && cwt.c != mouseover && !isNeighbor(cwt.c, mouseover) && mouseover) {
        teleportpc(mouseover);
        }
      else if(mousedest == -1)
        movepcto(mousedest);
      else
        movepcto((mousedest + 42 - cwt.spin)%42);
      }

    else if(andmode == 3) {
      showHelp(env, _thiz, help);
      }
    
    if(andmode == 4) {
      cmode = emQuit;
      }
    
    if(andmode == 10) {
      if(!playerfound) {
        centerpc(INF);
        View = Id;
        viewctr.h = cwt.c->master;
        }
      playermoved = true;
      }
      
    if(andmode >= 10) andmode -= 10;
    }
  
  graphdata.clear();
  drawscreen();
  
  if(clicked && lclicked && andmode == 1) {
    if(mouseoh[2] > .5 && mouseh[2] > .5 && mouseoh[2] < 50 && mouseh[2] < 50) {
      panning(mouseoh, mouseh);
      }
    }
  
  if(clicked && andmode == 2 && (mouseover != lmouseover || mouseovers != lmouseovers)) {
    addMessage(mouseovers);
    lmouseovers = mouseovers;
    }
  
  lclicked = clicked;
  delref;
  }
    
#ifdef FAKE

#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_gfxPrimitives.h>
#undef main

SDL_Surface *s;

int gdpos = 0;

int gdpop() { return graphdata[gdpos++]; }

TTF_Font *font[256];

bool rawdisplaystr(int x, int y, int shift, int size, const char *str, int color, int align) {

  if(strlen(str) == 0) return false;

  if(size <= 0 || size > 255) {
    return false;
    }
  SDL_Color col;
  col.r = (color >> 16) & 255;
  col.g = (color >> 8 ) & 255;
  col.b = (color >> 0 ) & 255;
  
  col.r >>= darken; col.g >>= darken; col.b >>= darken;

  if(!font[size])
    font[size] = TTF_OpenFont("VeraBd.ttf", size);

  SDL_Surface *txt = TTF_RenderText_Solid(font[size], str, col);
  
  if(txt == NULL) return false;

  SDL_Rect rect;

  rect.w = txt->w;
  rect.h = txt->h;

  rect.x = x - rect.w * align / 16;
  rect.y = y - rect.h/2;
  
  bool clicked = (mousex >= rect.x && mousey >= rect.y && mousex <= rect.x+rect.w && mousey <= rect.y+rect.h);
  
  SDL_BlitSurface(txt, NULL, s,&rect); 
  
  return clicked;
  }

#undef DEBT
void DEBT(const char *buf) {
  printf("%4d %s\n", SDL_GetTicks(), buf);
  }

char action;

int main(int argc, char **argv) {

  // printf("cell size = %d\n", int(sizeof(cell)));
  
  Java_com_roguetemple_hyperroid_HyperRogue_initGame(0, 0);
  
  vid.xres = 450; vid.yres = 600;
  // 450; vid.yres = 600;

  s= SDL_SetVideoMode(vid.xres, vid.yres, 32, 0);
  
  items[itOrbTeleport] = 100;

  if(TTF_Init() != 0) {
    printf("Failed to initialize TTF.\n");
    exit(2);
    }

  int mx = 0; int my = 0; bool clicked = false;
  
  while(true) {
   
          SDL_LockSurface(s);
          memset(s->pixels, 0, vid.xres * vid.yres * 4);
          SDL_UnlockSurface(s);

    Java_com_roguetemple_hyperroid_HyperRogue_update(0,0,
      vid.xres, vid.yres, SDL_GetTicks(), mx, my, clicked, action);
    action = 0;

    gdpos = 0;
    while(gdpos < graphdata.size()) {
      switch(gdpop()) {
        case 2: {
          int x = gdpop(), y = gdpop(), al = gdpop();
          int color = gdpop();
          int size = gdpop();
          int b = gdpop();
          int n = gdpop();
          string s;
          for(int i=0; i<n; i++) s += char(gdpop());
          rawdisplaystr(x, y, 0, size, s.c_str(), color, al);
          break;
          }
        
        case 1: {
          int col = gdpop();
          int num = gdpop();
          
          for(int i=0; i<num; i++) polyx[i] = gdpop(), polyy[i] = gdpop();

          filledPolygonColor(s, polyx, polyy, num, col);
          aapolygonColor(s, polyx, polyy, num, 0xFF);
          break;
          }

        case 3: {
          int col = gdpop();
          int num = gdpop();
          
          for(int i=0; i<num; i++) polyx[i] = gdpop(), polyy[i] = gdpop();
          
          for(int i=0; i<num-1; i++)
            aalineColor(s, polyx[i], polyy[i], polyx[i+1], polyy[i+1], col);
          }

        case 4: {
          int col = gdpop();
          int x = gdpop(), y = gdpop(), rad = gdpop();
          
          aacircleColor(s, x, y, rad, (col << 8) + 0xFF);
          }

        }
      }
    SDL_UpdateRect(s, 0, 0, vid.xres, vid.yres);

    SDL_Event ev;
    while(SDL_PollEvent(&ev)) {
    
      if(ev.type == SDL_MOUSEBUTTONDOWN) {
        mx = ev.button.x;
        my = ev.button.y;
        clicked = true;
        }

      if(ev.type == SDL_MOUSEBUTTONUP) {
        clicked = false;
        }

      if(ev.type == SDL_MOUSEMOTION) {
        mx = ev.motion.x;
        my = ev.motion.y;
        }
      
      if(ev.type == SDL_KEYDOWN) {
        int sym = ev.key.keysym.sym;
        if(sym == '1') {
  printf("Number of cells explored, by distance from the player:\n"); 
  for(int i=0; i<10; i++) printf(" %d", explore[i]); printf("\n");
          return 0;
          }
        if(sym == '2') {
          items[rand() % ittypes] += 3;
          kills[rand() % motypes] += 3;
          }
        if(sym == '3') {
          items[itHell] = 0;
          items[itGreenStone] = 100;
          }
        action = sym;
        }
        
      if(ev.type == SDL_QUIT) {
        SDL_Quit();
        return 0;
        }
      }
    }

  SDL_Quit();
  
  clearMemory();
  }

#endif


