package com.roguetemple.hyperroid;

import java.io.FileOutputStream;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Bundle;
import android.text.ClipboardManager;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.Toast;

public class HyperRogue extends Activity {

    HyperView hv;
    
    public void showHelp(String s) {
      AlertDialog ad = new AlertDialog.Builder(this).create();
      ad.setMessage(s);
      ad.setButton("OK", new DialogInterface.OnClickListener() {
        public void onClick(DialogInterface dialog, int which) {
                dialog.dismiss();
        }
      });
      ad.show();
      // help = s;
      // showDialog(1);
      }
      
    Bitmap bitmap;
    String sharescore;

    static final String[] items = {
      "share screenshot and score", "share score only", "copy score"
      };

    public void shareScore(String s) {
      sharescore = s;
      bitmap = hv.screenshot();

      AlertDialog.Builder builder = new AlertDialog.Builder(this);
      builder.setTitle("Share what and how?");
      builder.setItems(items, new DialogInterface.OnClickListener() {

        public void onClick(DialogInterface dialog, int item) {
          if(item == 2) {
            ClipboardManager man = (ClipboardManager)getSystemService(CLIPBOARD_SERVICE);
            if(man != null) man.setText(sharescore);
            Toast.makeText(getApplicationContext(), "Copied to Clipboard", Toast.LENGTH_SHORT).show();
            }
          else {
            Intent intent=new Intent(android.content.Intent.ACTION_SEND);
            intent.setType("text/plain");
            intent.putExtra(Intent.EXTRA_SUBJECT, "HyperRogue");
            intent.putExtra(Intent.EXTRA_TEXT, sharescore);
            intent.addFlags(Intent.FLAG_ACTIVITY_CLEAR_WHEN_TASK_RESET);
            if(item == 0) {
              try {
                String path = getCacheDir().toString() + "/cache.jpg";
                FileOutputStream fout = new FileOutputStream(path);
                bitmap.compress(Bitmap.CompressFormat.JPEG, 90, fout);
                fout.flush();
                fout.close();
                intent.setType("image/jpeg");
                intent.putExtra(Intent.EXTRA_STREAM, Uri.parse(HyperProvider.CONTENT_URI + "hyperrogue.jpg"));
                } catch (Exception e) {
                    Toast.makeText(getApplicationContext(), "Error with the image file", Toast.LENGTH_SHORT).show();
                }
              }
            startActivity(Intent.createChooser(intent, "Share via:"));
            }
          }
        });
      AlertDialog ad = builder.create();
      ad.setOwnerActivity(this);
      ad.show();
      }
      
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
      super.onCreate(savedInstanceState);
      setContentView(R.layout.main);
      
      initGame();

      hv = (HyperView) findViewById(R.id.ascv);
      hv.game = this;
      hv.invalidate();
      }
    
    public native int initGame();
    public native void update(int xres, int yres, int ticks, int mousex, int mousey, boolean clicked, char action);
    public native int[] loadMap();

    static {
      System.loadLibrary("hyper");
      }    

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
      MenuInflater inflater = getMenuInflater();
      inflater.inflate(R.menu.menugame, menu);
      return true;
      }
    
    public char action;
    
    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
      // Handle item selection
      switch (item.getItemId()) {
        case R.id.ascii:
          action = 'a';
          return true;
        case R.id.escher:
          action = 'e';
          return true;
        case R.id.black:
          action = 'b';
          return true;
        case R.id.tile:
          action = 'p';
          return true;
        case R.id.quest:
          action = 'q';
          return true;
        case R.id.restart:
          action = 'r';
          return true;
        case R.id.farsight:
          action = 'f';
          return true;
        case R.id.share:
          action = 's';
          return true;
        case R.id.reverse:
          action = 'w';
          return true;
        case R.id.help:
          action = 'h';
          return true;
        case R.id.klein:
            action = 'K';
            return true;
        case R.id.poincare:
            action = 'P';
            return true;
        case R.id.distant:
            action = 'X';
            return true;
        case R.id.website:
          openWebsite();
	  return true;
        default:
          return super.onOptionsItemSelected(item);
        }
      }

    void openWebsite() {
      Runnable r = new Runnable() {
        public void run() {
          String url = "http://roguetemple.com/z/hyper.php";
          Intent i = new Intent(Intent.ACTION_VIEW);
          i.setData(Uri.parse(url));
          startActivity(i);
          }
        };
      runOnUiThread(r);      
      }
    }
